from ipaddress import ip_address, ip_network

from rest_framework import authentication

from energy_base.authentications import MicroserviceUser


class MicroserviceUserAuthentication(authentication.BaseAuthentication):
    # List of local IP address ranges
    local_ip_ranges = [
        '127.0.0.0/8',  # Loopback addresses
        '10.0.0.0/8',  # Private network addresses
        '172.16.0.0/12',  # Private network addresses
        '192.168.0.0/16',  # Private network addresses
        # Add more local ranges if needed
    ]

    def authenticate(self, request):
        client_ip = request.META.get('REMOTE_ADDR', None)
        server_ip = request.META.get('SERVER_ADDR', None)

        if any(ip_address(client_ip) in ip_network(ip_range, strict=False) for ip_range in self.local_ip_ranges):
            return self.get_user(), None

        if client_ip == server_ip:
            return self.get_user(), None

    def get_user(self) -> MicroserviceUser:
        return MicroserviceUser()



from django_filters import rest_framework as filters

from ..models import ImportFile


class FileFilterSet(filters.FilterSet):
    active = filters.BooleanFilter(field_name='active')
    created_at = filters.DateFilter(lookup_expr='date')

    class Meta:
        model = ImportFile
        fields = ['active', 'created_at']



def compute_difference_percent(a, b):
    return 100 * (a or 0) / (b or 1)

def compute_total_and_difference(total_given_date, total_given_date_prev):
    total_given_date = total_given_date or 0
    total_given_date_prev = total_given_date_prev or 0
    return {
        'total': total_given_date,
        'difference': total_given_date - total_given_date_prev,
        'difference_percent': 0 if not total_given_date and not total_given_date_prev else 100 * total_given_date / (
            total_given_date_prev or 1) - 100
    }



import requests
from rest_framework.request import Request

from .constants import AOServices
from .models import AOService


def get(request: Request, ao_service: AOServices, prefix: str, params: any = None):
    headers = {
        'Authorization': request.headers.get('Authorization'),
        'Accept-Language': request.headers.get('Accept-Language'),
    }
    return requests.get(
        url=AOService.get_by_name(ao_service).get_url(prefix),
        params=params,
        headers=headers,
    )


def post(request: Request, ao_service: AOServices, prefix: str, params: any = None, data: any = None):
    headers = {
        'Authorization': request.headers.get('Authorization'),
        'Accept-Language': request.headers.get('Accept-Language'),
    }
    return requests.post(
        url=AOService.get_by_name(ao_service).get_url(prefix),
        params=params,
        headers=headers,
        data=data,
    )



from django.db import models


class AOServices:
    GES = 'ges'
    IES = 'ies'
    HET = 'het'
    MET = 'MET'
    HUDUDGAZ_LIQUEFIED_GAS = 'hududgaz-liquefied-gas'
    HUDUDGAZ_NATURAL_GAS = 'hududgaz-natural-gas'
    NEFTEGAZ_LIQUID_HYDROCARBONS = 'liquid-hydrocarbons'
    UZBEKNEFTEGAZ_OIL_WELL = 'oil-well'
    UZBEKNEFTEGAZ_NATURAL_GAZ = 'uzbekneftegaz'
    UZ_GAZ_TRADE = 'uz-gas-trade'
    UZ_TRANS_GAZ = 'uz-trans-gaz'
    UZBEKKUMIR = 'uzbekkumir'
    TRANSLATION = 'translation'
    USER_MANAGEMENT = 'user-management'
    AO_SERVICE = 'ao-service'

class RegionShortCode(models.TextChoices):
    ANDIJON = 'AN'
    BUXORO = 'BX'
    FARGONA = 'FR'
    JIZZAX = 'JZ'
    NAMANGAN = 'NM'
    NAVOIY = 'NV'
    QASHQADARYO = 'QD'
    QORAQALPOGISTON = 'QR'
    SAMARQAND = 'SN'
    SIRDARYO = 'SR'
    SURXONDARYO = 'SD'
    TOSHKENT_CITY = 'TN'
    TOSHKENT = 'TV'
    XORAZM = 'XR'



from django.middleware.cache import UpdateCacheMiddleware, FetchFromCacheMiddleware


class CachePermissionMixin:
    # noinspection PyMethodMayBeStatic
    def should_cache(self, request):
        user = request.user
        if not user.is_authenticated:
            return True
        if hasattr(user, 'should_cache_requests') and not user.should_cache_requests:
            return False
        return True


class EnergyBaseUpdateCacheMiddleware(CachePermissionMixin, UpdateCacheMiddleware):
    def process_response(self, request, response):
        if not self.should_cache(request):
            return response
        return super().process_response(request, response)


class EnergyBaseFetchFromCacheMiddleware(CachePermissionMixin, FetchFromCacheMiddleware):
    def process_request(self, request):
        if not self.should_cache(request):
            return None
        return super().process_request(request)




from ...services import MinioService

from django.conf import settings

minio_service = MinioService(
    endpoint=settings.MINIO.get("ENDPOINT"),
    access_key=settings.MINIO.get("ACCESS_KEY"),
    secret_key=settings.MINIO.get("SECRET_KEY"),
)



def get_by_dotted(dict_obj: dict, dotted_key: str, default=None):
    """
    Get a value from a dictionary using a dotted key notation.

    Args:
        dict_obj (dict): The dictionary to retrieve the value from.
        dotted_key (str): The dotted key notation to access nested keys.
        default: The default value to return if the key is not found.

    Returns:
        The value at the given dotted key, or default if not found.
    """
    parts = dotted_key.split('.')
    for part in parts:
        # Check if current part is in the dictionary and is a dictionary itself
        if isinstance(dict_obj, dict) and part in dict_obj:
            dict_obj = dict_obj.get(part, default)
        else:
            return default
    return dict_obj



from django_q.models import Task
from rest_framework import serializers


class TaskSerializer(serializers.ModelSerializer):
    class Meta:
        model = Task
        fields = '__all__'

    def to_representation(self, instance):
        if isinstance(instance, str):
            instance = Task.objects.filter(pk=instance).first()

        if not instance:
            return None
        data = super(TaskSerializer, self).to_representation(instance)
        data['args'] = instance.args
        data['kwargs'] = instance.kwargs
        data['result'] = instance.result

        return data



from openpyxl.workbook import Workbook
from rest_framework.exceptions import ValidationError

from ..models import ImportFile
from energy_base.translation import translate as _


class BaseSheetProcessor:
    sheet_index = 0

    def get_cell(self, wb: Workbook, cell: str):
        return wb.worksheets[self.sheet_index][cell] or 0

    def get_row(self, wb: Workbook, from_column: str, to_column: str):
        return [c[0].value or 0 for c in wb.worksheets[self.sheet_index][f"{from_column}:{to_column}"]]

    def get_column(self, wb: Workbook, from_column: str, to_column: str):
        return [c[0].value or 0 for c in wb.worksheets[self.sheet_index][f"{from_column}:{to_column}"]]

    def get_matrix(self, wb: Workbook, from_cell: str, to_cell: str):
        try:
            return [[c.value or 0 for c in r] for r in wb.worksheets[self.sheet_index][f"{from_cell}:{to_cell}"]]
        except:
            raise ValidationError(_('Wrong file was uploaded'), code='file')

    def _validate_matrix(self, wb: Workbook, from_cell: str, to_cell: str):
        errors = []
        data = self.get_matrix(wb, from_cell, to_cell)
        cell_code = from_cell
        for i, row in enumerate(data):
            for j, cell in enumerate(row):
                if not isinstance(cell, (float, int)):
                    error_cell = chr(ord(cell_code[0]) + j) + str(int(cell_code[1]) + i)
                    errors.append(f'{error_cell}')

        return errors

    def get_data(self, wb: Workbook) -> list:
        raise NotImplementedError

    def validate(self, wb: Workbook) -> dict | None:
        raise NotImplementedError

    def write_db(self, data: list, file: ImportFile, created_by: str) -> bool:
        raise NotImplementedError



from django.urls import path

from .views import (
    ImportFileViewSet,
    FileDownloadViewSet,
    FileUploadViewSet,
    FileConfirmViewSet,
    FileUploadStatusView
)

from ..api.routers import OptionalSlashRouter

router = OptionalSlashRouter()
router.register('files', ImportFileViewSet, 'files')

urlpatterns = [
    path('files/<pk>/download/', FileDownloadViewSet.as_view(), name='download'),
    path('files/upload/', FileUploadViewSet.as_view(), name='upload'),
    path('files/<pk>/confirm/', FileConfirmViewSet.as_view(), name='confirm'),
    path('file-upload-status/', FileUploadStatusView.as_view(), name='file-upload-status'),
]

urlpatterns += router.urls



from energy_base.q_api.views.schedule import ScheduleViewSet, ScheduleRunNowView
from energy_base.q_api.views.task import TaskViewSet

__all__ = [
    'TaskViewSet',
    'ScheduleViewSet',
    'ScheduleRunNowView'
]



import django_filters
from django_q.models import Task
from drf_spectacular.utils import extend_schema
from rest_framework import viewsets, filters

from energy_base.q_api.filters import TaskFilterSet
from energy_base.api.permissions import IsSuperUser
from energy_base.q_api.serializers import TaskSerializer


@extend_schema(tags=['admin/tasks'])
class TaskViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Task.objects.all()
    serializer_class = TaskSerializer
    permission_classes = [IsSuperUser]
    filterset_class = TaskFilterSet
    filter_backends = [filters.SearchFilter, django_filters.rest_framework.DjangoFilterBackend]
    search_fields = ['name', 'group']
